@@ -2,10 +2,11 @@ import json
 import uuid
 from datetime import datetime, timedelta
 from typing import Callable
-from urllib.parse import parse_qs, urlparse
+from urllib.parse import parse_qs, urljoin, urlparse
 
 import pystac
 import pytest
+from httpx import AsyncClient
 from shapely.geometry import Polygon
 from stac_pydantic import Collection, Item
 from stac_pydantic.shared import DATETIME_RFC339
@@ -67,7 +68,7 @@ async def test_create_item(app_client, load_test_data: Callable, load_test_colle
     in_json = load_test_data("test_item.json")
     in_item = Item.parse_obj(in_json)
     resp = await app_client.post(
-        "/collections/{coll.id}/items",
+        f"/collections/{coll.id}/items",
         json=in_json,
     )
     assert resp.status_code == 200
@@ -365,6 +366,13 @@ async def test_item_search_spatial_query_post(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     params = {
         "collections": [test_item["collection"]],
         "intersects": test_item["geometry"],
@@ -372,6 +380,7 @@ async def test_item_search_spatial_query_post(
     resp = await app_client.post("/search", json=params)
     assert resp.status_code == 200
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -386,6 +395,13 @@ async def test_item_search_temporal_query_post(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     item_date = datetime.strptime(test_item["properties"]["datetime"], DATETIME_RFC339)
     print(item_date)
     item_date = item_date + timedelta(seconds=1)
@@ -399,6 +415,7 @@ async def test_item_search_temporal_query_post(
     resp = await app_client.post("/search", json=params)
     print(resp.content)
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -413,17 +430,24 @@ async def test_item_search_temporal_window_post(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     item_date = datetime.strptime(test_item["properties"]["datetime"], DATETIME_RFC339)
     item_date_before = item_date - timedelta(seconds=1)
     item_date_after = item_date + timedelta(seconds=1)
 
     params = {
         "collections": [test_item["collection"]],
-        "intersects": test_item["geometry"],
         "datetime": f"{item_date_before.strftime(DATETIME_RFC339)}/{item_date_after.strftime(DATETIME_RFC339)}",
     }
     resp = await app_client.post("/search", json=params)
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -438,14 +462,20 @@ async def test_item_search_temporal_open_window(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     params = {
         "collections": [test_item["collection"]],
-        "intersects": test_item["geometry"],
         "datetime": "../..",
     }
     resp = await app_client.post("/search", json=params)
     resp_json = resp.json()
-    assert resp_json["features"][0]["id"] == test_item["id"]
+    assert len(resp_json["features"]) == 2
 
 
 @pytest.mark.asyncio
@@ -507,6 +537,13 @@ async def test_item_search_bbox_get(app_client, load_test_data, load_test_collec
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     params = {
         "collections": test_item["collection"],
         "bbox": ",".join([str(coord) for coord in test_item["bbox"]]),
@@ -514,6 +551,7 @@ async def test_item_search_bbox_get(app_client, load_test_data, load_test_collec
     resp = await app_client.get("/search", params=params)
     assert resp.status_code == 200
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -528,12 +566,20 @@ async def test_item_search_get_without_collections(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     params = {
         "bbox": ",".join([str(coord) for coord in test_item["bbox"]]),
     }
     resp = await app_client.get("/search", params=params)
     assert resp.status_code == 200
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -548,17 +594,24 @@ async def test_item_search_temporal_window_get(
     )
     assert resp.status_code == 200
 
+    # Add second item with a different datetime.
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     item_date = datetime.strptime(test_item["properties"]["datetime"], DATETIME_RFC339)
     item_date_before = item_date - timedelta(seconds=1)
     item_date_after = item_date + timedelta(seconds=1)
 
     params = {
         "collections": test_item["collection"],
-        "bbox": ",".join([str(coord) for coord in test_item["bbox"]]),
         "datetime": f"{item_date_before.strftime(DATETIME_RFC339)}/{item_date_after.strftime(DATETIME_RFC339)}",
     }
     resp = await app_client.get("/search", params=params)
     resp_json = resp.json()
+    assert len(resp_json["features"]) == 1
     assert resp_json["features"][0]["id"] == test_item["id"]
 
 
@@ -599,6 +652,12 @@ async def test_item_search_post_without_collection(
     )
     assert resp.status_code == 200
 
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     params = {
         "bbox": test_item["bbox"],
     }
@@ -619,6 +678,12 @@ async def test_item_search_properties_jsonb(
     )
     assert resp.status_code == 200
 
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     # EPSG is a JSONB key
     params = {"query": {"proj:epsg": {"gt": test_item["properties"]["proj:epsg"] - 1}}}
     print(params)
@@ -639,7 +704,13 @@ async def test_item_search_properties_field(
     )
     assert resp.status_code == 200
 
-    params = {"query": {"platform": {"eq": "landsat-8"}}}
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
+    params = {"query": {"eo:cloud_cover": {"eq": 0}}}
     print(params)
     resp = await app_client.post("/search", json=params)
     assert resp.status_code == 200
@@ -658,6 +729,12 @@ async def test_item_search_get_query_extension(
     )
     assert resp.status_code == 200
 
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     # EPSG is a JSONB key
     params = {
         "collections": [test_item["collection"]],
@@ -693,6 +770,12 @@ async def test_item_search_get_filter_extension_cql(
     )
     assert resp.status_code == 200
 
+    second_test_item = load_test_data("test_item2.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=second_test_item
+    )
+    assert resp.status_code == 200
+
     # EPSG is a JSONB key
     params = {
         "collections": [test_item["collection"]],
@@ -1005,3 +1088,168 @@ async def test_search_bbox_errors(app_client):
     params = {"bbox": "100.0,0.0,0.0,105.0"}
     resp = await app_client.get("/search", params=params)
     assert resp.status_code == 400
+
+
+@pytest.mark.asyncio
+async def test_preserves_extra_link(
+    app_client: AsyncClient, load_test_data, load_test_collection
+):
+    coll = load_test_collection
+    test_item = load_test_data("test_item.json")
+    expected_href = urljoin(str(app_client.base_url), "preview.html")
+
+    resp = await app_client.post(f"/collections/{coll.id}/items", json=test_item)
+    assert resp.status_code == 200
+
+    response_item = await app_client.get(
+        f"/collections/{coll.id}/items/{test_item['id']}",
+        params={"limit": 1},
+    )
+    assert response_item.status_code == 200
+    item = response_item.json()
+
+    extra_link = [link for link in item["links"] if link["rel"] == "preview"]
+    assert extra_link
+    assert extra_link[0]["href"] == expected_href
+
+
+@pytest.mark.asyncio
+async def test_item_search_get_filter_extension_cql_explicitlang(
+    app_client, load_test_data, load_test_collection
+):
+    """Test GET search with JSONB query (cql json filter extension)"""
+    test_item = load_test_data("test_item.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=test_item
+    )
+    assert resp.status_code == 200
+
+    # EPSG is a JSONB key
+    params = {
+        "collections": [test_item["collection"]],
+        "filter-lang": "cql-json",
+        "filter": {
+            "gt": [
+                {"property": "proj:epsg"},
+                test_item["properties"]["proj:epsg"] + 1,
+            ]
+        },
+    }
+    resp = await app_client.post("/search", json=params)
+    resp_json = resp.json()
+
+    assert resp.status_code == 200
+    assert len(resp_json.get("features")) == 0
+
+    params = {
+        "collections": [test_item["collection"]],
+        "filter-lang": "cql-json",
+        "filter": {
+            "eq": [
+                {"property": "proj:epsg"},
+                test_item["properties"]["proj:epsg"],
+            ]
+        },
+    }
+    resp = await app_client.post("/search", json=params)
+    resp_json = resp.json()
+    assert len(resp.json()["features"]) == 1
+    assert (
+        resp_json["features"][0]["properties"]["proj:epsg"]
+        == test_item["properties"]["proj:epsg"]
+    )
+
+
+@pytest.mark.asyncio
+async def test_item_search_get_filter_extension_cql2(
+    app_client, load_test_data, load_test_collection
+):
+    """Test GET search with JSONB query (cql json filter extension)"""
+    test_item = load_test_data("test_item.json")
+    resp = await app_client.post(
+        f"/collections/{test_item['collection']}/items", json=test_item
+    )
+    assert resp.status_code == 200
+
+    # EPSG is a JSONB key
+    params = {
+        "filter-lang": "cql2-json",
+        "filter": {
+            "op": "and",
+            "args": [
+                {
+                    "op": "eq",
+                    "args": [
+                        {"property": "proj:epsg"},
+                        test_item["properties"]["proj:epsg"] + 1,
+                    ],
+                },
+                {
+                    "op": "in",
+                    "args": [
+                        {"property": "collection"},
+                        [test_item["collection"]],
+                    ],
+                },
+            ],
+        },
+    }
+    print(json.dumps(params))
+    resp = await app_client.post("/search", json=params)
+    resp_json = resp.json()
+    print(resp_json)
+
+    assert resp.status_code == 200
+    assert len(resp_json.get("features")) == 0
+
+    params = {
+        "filter-lang": "cql2-json",
+        "filter": {
+            "op": "and",
+            "args": [
+                {
+                    "op": "eq",
+                    "args": [
+                        {"property": "proj:epsg"},
+                        test_item["properties"]["proj:epsg"],
+                    ],
+                },
+                {
+                    "op": "in",
+                    "args": [
+                        {"property": "collection"},
+                        [test_item["collection"]],
+                    ],
+                },
+            ],
+        },
+    }
+    resp = await app_client.post("/search", json=params)
+    resp_json = resp.json()
+    print(resp_json)
+    assert len(resp.json()["features"]) == 1
+    assert (
+        resp_json["features"][0]["properties"]["proj:epsg"]
+        == test_item["properties"]["proj:epsg"]
+    )
+
+
+@pytest.mark.asyncio
+async def test_search_datetime_validation_errors(app_client):
+    bad_datetimes = [
+        "37-01-01T12:00:27.87Z",
+        "1985-13-12T23:20:50.52Z",
+        "1985-12-32T23:20:50.52Z",
+        "1985-12-01T25:20:50.52Z",
+        "1985-12-01T00:60:50.52Z",
+        "1985-12-01T00:06:61.52Z",
+        "1990-12-31T23:59:61Z",
+        "1986-04-12T23:20:50.52Z/1985-04-12T23:20:50.52Z",
+    ]
+    for dt in bad_datetimes:
+        body = {"query": {"datetime": dt}}
+        resp = await app_client.post("/search", json=body)
+        assert resp.status_code == 400
+
+        resp = await app_client.get("/search?datetime={}".format(dt))
+        assert resp.status_code == 400
